package javax.management;

import java.security.AccessController;
import com.sun.jmx.mbeanserver.GetPropertyAction;
import java.util.Objects;
import com.sun.jmx.mbeanserver.Introspector;
import java.lang.reflect.Method;

public class MBeanAttributeInfo extends MBeanFeatureInfo implements Cloneable
{
    private static final long serialVersionUID;
    static final MBeanAttributeInfo[] NO_ATTRIBUTES;
    private final String attributeType;
    private final boolean isWrite;
    private final boolean isRead;
    private final boolean is;
    
    public MBeanAttributeInfo(String s, String s2, String s3, boolean b, boolean b2, boolean b3) {
        this(s, s2, s3, b, b2, b3, (Descriptor)null);
    }
    
    public MBeanAttributeInfo(String s, String attributeType, String s2, boolean isRead, boolean isWrite, boolean is, Descriptor descriptor) {
        super(s, s2, descriptor);
        this.attributeType = attributeType;
        this.isRead = isRead;
        this.isWrite = isWrite;
        if (is && !isRead) {
            throw new IllegalArgumentException("Cannot have an \"is\" getter for a non-readable attribute");
        }
        if (is && !attributeType.equals("java.lang.Boolean") && !attributeType.equals("boolean")) {
            throw new IllegalArgumentException("Cannot have an \"is\" getter for a non-boolean attribute");
        }
        this.is = is;
        return;
    }
    
    public MBeanAttributeInfo(String s, String s2, Method elmt, Method elmt2) throws IntrospectionException {
        this(s, attributeType(elmt, elmt2), s2, elmt != null, elmt2 != null, isIs(elmt), ImmutableDescriptor.union(new Descriptor[] { Introspector.descriptorForElement(elmt), Introspector.descriptorForElement(elmt2) }));
    }
    
    public Object clone() {
        try {
            return super.clone();
        }
        catch (CloneNotSupportedException ex) {
            return null;
        }
    }
    
    public String getType() {
        return this.attributeType;
    }
    
    public boolean isReadable() {
        return this.isRead;
    }
    
    public boolean isWritable() {
        return this.isWrite;
    }
    
    public boolean isIs() {
        return this.is;
    }
    
    @Override
    public String toString() {
        String str;
        if (this.isReadable()) {
            if (this.isWritable()) {
                str = "read/write";
            }
            else {
                str = "read-only";
            }
        }
        else if (this.isWritable()) {
            str = "write-only";
        }
        else {
            str = "no-access";
        }
        return this.getClass().getName() + "[description=" + this.getDescription() + ", name=" + this.getName() + ", type=" + this.getType() + ", " + str + ", " + (this.isIs() ? "isIs, " : "") + "descriptor=" + this.getDescriptor() + "]";
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof MBeanAttributeInfo)) {
            return false;
        }
        MBeanAttributeInfo mBeanAttributeInfo = (MBeanAttributeInfo)o;
        return Objects.equals(mBeanAttributeInfo.getName(), this.getName()) && Objects.equals(mBeanAttributeInfo.getType(), this.getType()) && Objects.equals(mBeanAttributeInfo.getDescription(), this.getDescription()) && Objects.equals(mBeanAttributeInfo.getDescriptor(), this.getDescriptor()) && mBeanAttributeInfo.isReadable() == this.isReadable() && mBeanAttributeInfo.isWritable() == this.isWritable() && mBeanAttributeInfo.isIs() == this.isIs();
    }
    
    @Override
    public int hashCode() {
        return Objects.hash(new Object[] { this.getName(), this.getType() });
    }
    
    private static boolean isIs(Method method) {
        return method != null && method.getName().startsWith("is") && (method.getReturnType().equals(Boolean.TYPE) || method.getReturnType().equals(Boolean.class));
    }
    
    private static String attributeType(Method method, Method method2) throws IntrospectionException {
        Class<Object> returnType = null;
        if (method != null) {
            if (method.getParameterTypes().length != 0) {
                throw new IntrospectionException("bad getter arg count");
            }
            returnType = method.getReturnType();
            if (returnType == Void.TYPE) {
                throw new IntrospectionException("getter " + method.getName() + " returns void");
            }
        }
        if (method2 != null) {
            Class[] parameterTypes = method2.getParameterTypes();
            if (parameterTypes.length != 1) {
                throw new IntrospectionException("bad setter arg count");
            }
            if (returnType == null) {
                returnType = parameterTypes[0];
            }
            else if (returnType != parameterTypes[0]) {
                throw new IntrospectionException("type mismatch between getter and setter");
            }
        }
        if (returnType == null) {
            throw new IntrospectionException("getter and setter cannot both be null");
        }
        return returnType.getName();
    }
    
    static {
        long serialVersionUID2 = 8644704819898565848L;
        try {
            if ("1.0".equals((String)AccessController.doPrivileged(new GetPropertyAction("jmx.serial.form")))) {
                serialVersionUID2 = 7043855487133450673L;
            }
        }
        catch (Exception ex) {}
        serialVersionUID = serialVersionUID2;
        NO_ATTRIBUTES = new MBeanAttributeInfo[0];
    }
}
